#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vPosition; // vertex position (in model space)
layout(location = 1) in vec3 vNormal; // vertex normal (in model space)

// Output data will be interpolated for each fragment.
out vec3 PositionWorldPass;
out vec3 NormalWorldPass;

// Values that stay constant for the whole mesh.
uniform mat4 gModel, gView, gProjection;

void main()
{
	// Output position of the vertex, in clip space : gMVP * position
	mat4 MVP = gProjection * gView * gModel;
	gl_Position =  MVP * vec4(vPosition,1);
	
	// Position of the vertex, in worldspace : gWorld * position
	PositionWorldPass = (gModel * vec4(vPosition,1)).xyz;
	
	// Vertex normal vector in world space
	NormalWorldPass = (gModel * vec4(vNormal, 1)).xyz;
}

