#include "moglclass.h"
#include "shaders.h"
#include "geometryPositionNormal.h"
#include "simpleCar.h"
#include "simpleSolar.h"
#include "simpleRobot.h"
#include "simpleMobile.h"
#include "simpleCube3d.h"
#include "simpleTree.h"
#include "light.h"
#include "camera.h"

using namespace moglclass;

#define NUMSHADERS 4

Shader shShaders[NUMSHADERS];
Program spMain;

Light light0(glm::vec4(-1.0f, 0.0f, 0.0f, 0.0f), glm::vec4(0.2f, 0.2f, 0.2f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 1.0f, 0.0f, 0.0f, 180.0f, 0.0f, glm::vec3(0.0f, 0.0f, 0.0f)); // directional light
Light light1(glm::vec4(5.0f, 5.0f, 4.0f, 1.0f), glm::vec4(0.2f, 0.2f, 0.2f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 1.0f, 0.0f, 0.0f, 180.0f, 0.0f, glm::vec3(0.0f, 0.0f, 0.0f)); // point light
Light light2(glm::vec4(-5.0f, -5.0f, 4.0f, 1.0f), glm::vec4(0.2f, 0.2f, 0.2f, 1.0f), glm::vec4(1.0f, 1.0f, 0.0f, 1.0f), glm::vec4(1.0f, 1.0f, 0.0f, 1.0f), 0.0f, 1.0f, 0.0f, 60.0f, 50.0f, glm::vec3(0.0f, 0.0f, -1.0f)); // spot light
Material material1(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);
Material material2(glm::vec4(1.0f, 0.0f, 0.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);

std::vector<GeometryPositionNormal*> geoList;
SimpleCar* car;
SimpleSolar* solar;
SimpleRobot* robot;
SimpleMobile* mobile;
SimpleCube3d* cube;
SimpleTree* tree;

bool g_wiremode = true;
int g_geometrymode = 0;
int startTime = 0;

float g_fovy = 45.0f;
float g_aspect = 1.0f;
float g_zNear = 1.0f;
float g_zFar = 100.f;

Camera camera1(FLY);
float cameraHeading = 0.0f;
float cameraZ = 10.0f;
glm::vec3 g_eye(0, 0, 10);
glm::vec3 g_at(0, 0, 0);
glm::vec3 g_up(0, 1, 0);

glm::mat4 Identity(1.0f);
glm::mat4 World(1.0f);
glm::mat4 View(1.0f);
glm::mat4 Projection(1.0f);
glm::mat4 MVP(1.0f);

glm::mat4 yRotation(1.0f);
glm::mat4 yRotation2(1.0f);
glm::mat4 yRotation3(1.0f);
glm::mat4 mTranslate(1.0f);

float g_theta = 0.0f;
float g_phi = 0.0f;
float g_psi = 0.0f;

void init( void )
{
    // load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("MPerPixelShading.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("MPerPixelShading.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();
	
	Projection = glm::perspective(g_fovy, g_aspect, g_zNear, g_zFar);
	View = camera1.lookAt(g_eye, g_at, g_up);

	// set geometry
	geoList.push_back(new Cube(glm::vec3(0, 0, 0), 5.0f));
	geoList.push_back(new Grid(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(0.0f, 1.0f, 0.0f), 8.0f, 6.0f, 4, 3));
	geoList.push_back(new Sphere(glm::vec3(0, 0, 0), 2.5f, 16, 16));
	geoList.push_back(new Cylinder(glm::vec3(0, 0, 0), 2.5f, 4.0f, 16));
	geoList.push_back(new Torus(glm::vec3(0, 0, 0), 3.0f, 1.0f, 32, 16));
	geoList.push_back(new Parallelepiped(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(3.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 3.0f), glm::vec3(4.5f, 7.0f, 0.0f)));

	car = new SimpleCar();
	robot = new SimpleRobot();
	solar = new SimpleSolar();
	mobile = new SimpleMobile();
	cube = new SimpleCube3d();
	tree = new SimpleTree();

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
}

void cleanup()
{		 
	for (std::vector<GeometryPositionNormal*>::iterator i = geoList.begin(); i != geoList.end(); ++i)
	{
		delete (*i);
		i = geoList.erase(i);
	}

	if (car) delete car;
	if (robot) delete robot;
	if (solar) delete solar;
	if (mobile) delete mobile;
	if (cube) delete cube;
	if (tree) delete tree;
}

void display( void )
{
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );     // clear the window

	// set drawmode
	if (g_wiremode) 
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  // draw wireframe 
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	// draw filled 

	// MVP matrix
	View = camera1.View();
	World = yRotation;
	MVP = Projection * View * World;
	spMain.useProgram();
	spMain.setUniform("gMVP", MVP);
	spMain.setUniform("gProjection", Projection);
	spMain.setUniform("gView", View);
	spMain.setUniform("gModel", World);

	light0.setUniform(&spMain, "gDirectionalLight");
	light1.setUniform(&spMain, "gPointLight");
	light2.setUniform(&spMain, "gSpotLight");

	glm::vec3 cameraPos = camera1.getPosition();
	spMain.setUniform("gEyePosition", cameraPos);
	//glm::vec3 lightPos = glm::vec3(4,4,4);
	//spMain.setUniform("gLightPosition", lightPos);

	// draw geometry
	switch (g_geometrymode)
	{
	case 0:
		Material::SkyBlueMaterial.setUniform(&spMain, "gMaterial");
		geoList[0]->draw();;
		break;
	case 1:
		Material::NavyBlueMaterial.setUniform(&spMain, "gMaterial");
		geoList[1]->draw();;
		break;
	case 2:
		Material::YellowGreenMaterial.setUniform(&spMain, "gMaterial");
		geoList[2]->draw();;
		break;
	case 3:
		Material::MaroonMaterial.setUniform(&spMain, "gMaterial");
		geoList[3]->draw();;
		break;
	case 4:
		Material::LavenderMaterial.setUniform(&spMain, "gMaterial");
		geoList[4]->draw();;
		break;
	case 5:
		Material::OrangeMaterial.setUniform(&spMain, "gMaterial");
		geoList[5]->draw();;
		break;
	case 6:
		car->draw(&spMain, Projection, View, Identity);
		break;
	case 7:
		solar->draw(&spMain, Projection, View, Identity);
		break;
	case 8:
		robot->draw(&spMain, Projection, View, Identity);
		break;
	case 9:
		mobile->draw(&spMain, Projection, View, World);
		break;
	case 10:
		cube->draw(&spMain, Projection, View, World);
		break;
	case 11:
		tree->draw(&spMain, Projection, View, World);
		break;
	}

	glBindVertexArray(0);
	glDisableVertexAttribArray(0);

    glutSwapBuffers();
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0; 
	static int elapsedTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;
	prevTime = currentTime;

    // translate & rotate around x/y axis
	static float angle = 0.0f;
	angle += 0.001f;

	// geometry: translate & rotate in x & rotate in y
	glm::mat4 T = glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, 1));
	glm::mat4 R = glm::rotate(T, angle, glm::vec3(1, 0, 0));
	yRotation = glm::rotate(R, angle, glm::vec3(0, 1, 0));

	// each geometry update
	if (g_geometrymode >= 0 && g_geometrymode <= 5) {
		geoList[g_geometrymode]->update((float)deltaTime);
	}
	car->update((float)deltaTime);
	robot->update((float)deltaTime);
	solar->update((float)deltaTime);
	mobile->update((float)deltaTime);
	tree->update((float)deltaTime);
	cube->update((float)elapsedTime); // cube elapsedTime 

	elapsedTime = currentTime - startTime;
	glutPostRedisplay();
}
void keyboard( unsigned char key, int x, int y )
{
	switch (key)  
	{    
	case 0x1B:    
	case 'q':    
	case 'Q':      
		exit(0);     
	case ' ':
		g_wiremode = !g_wiremode;
		break;  
	case 'g':
		g_geometrymode = (g_geometrymode+1) % 12;
		printf("g_geometrymode=%d\n", g_geometrymode);
		if (g_geometrymode == 6) {
			car->activate();
		}
		else if (g_geometrymode == 7) {
			solar->activate();
		}
		else if (g_geometrymode == 8) {
			robot->activate();
		}
		else if (g_geometrymode == 9) {
			mobile->activate();
		}
		else if (g_geometrymode == 10) {
			startTime = glutGet(GLUT_ELAPSED_TIME); // cube animation
			cube->activate();
		}
		break;
	case 'l':
		light2.Position[0] += 1.0f;
		light2.Position[1] += 1.0f;
		light2.Position[2] += 1.0f;
		break;
	case 'k':
		light2.Position[0] -= 1.0f;
		light2.Position[1] -= 1.0f;
		light2.Position[2] -= 1.0f;
		break;
	}
	glutPostRedisplay();
}

void specialkey(int key, int x, int y)
{
 if (key == GLUT_KEY_F1)   // x-movement
        camera1.strafe(0.5);
 else if (key == GLUT_KEY_F2)
        camera1.strafe(-0.5);
 else if (key == GLUT_KEY_F3) // y-movement
        camera1.fly(0.5);
 else if (key == GLUT_KEY_F4)
        camera1.fly(-0.5);
 else if (key == GLUT_KEY_F5) // z-movement
        camera1.walk(0.5);
    else if (key == GLUT_KEY_F6)
        camera1.walk(-0.5);

 else if (key == GLUT_KEY_F7) // yaw (by y-axis)
        camera1.yaw(2.5);
 else if (key == GLUT_KEY_F8)
        camera1.yaw(-2.5);
 else if (key == GLUT_KEY_F9) // pitch (by x-axis)
        camera1.pitch(2.5);
    else if (key == GLUT_KEY_F10)
        camera1.pitch(-2.5);
 else if (key == GLUT_KEY_HOME) // roll (by z-axis)
        camera1.roll(2.5);
    else if (key == GLUT_KEY_END)
        camera1.roll(-2.5);

 else if (key == GLUT_KEY_LEFT) // same as town
        camera1.yaw(2.5);
 else if (key == GLUT_KEY_RIGHT)
        camera1.yaw(-2.5);
 else if (key == GLUT_KEY_UP) 
        camera1.walk(0.5);
    else if (key == GLUT_KEY_DOWN)
        camera1.walk(-0.5);

 glutPostRedisplay();
}


void mouse(int button, int state, int mx, int my)
{
	if (button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN)
		exit(0);
}

void motion(int mx, int my)
{
	int w = glutGet(GLUT_WINDOW_WIDTH);
	int h = glutGet(GLUT_WINDOW_HEIGHT);
	float x = (float) (mx - w*0.5) / w;
	float y = (float) (h*0.5 - my) / h;
	//fprintf(stderr, "x=%f y=%f\n", x, y);
	mTranslate = glm::translate(glm::mat4(1.0f), glm::vec3(x, y, 0)); 

	glutPostRedisplay();
}

void reshape(int w, int h)
{
	g_aspect = (float) (w/h);

    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glutPostRedisplay();
}

int main( int argc, char **argv )
{
	glutInit(&argc, argv);
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
    glutInitWindowSize( SCREEN_WIDTH, SCREEN_HEIGHT );
    glutCreateWindow(argv[0]);
    
	glewInit();
    init();

    glutDisplayFunc( display );
    glutKeyboardFunc( keyboard );
    glutSpecialFunc( specialkey );
	glutMouseFunc( mouse ); 
	glutMotionFunc( motion ); 
	glutIdleFunc( update );
	glutReshapeFunc( reshape );

    glutMainLoop();

	cleanup();

    return 0;
}
