// material class
// -Kyoung Shin Park (2016 Fall)
//////////////////////////////////////////////////////////////////////
#include "material.h"

namespace moglclass
{
const Material Material::WhiteMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);
const Material Material::BlackMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(0.0f, 0.0f, 0.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);
const Material Material::RedMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(1.0f, 0.0f, 0.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);
const Material Material::GreenMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(0.0f, 1.0f, 0.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);
const Material Material::BlueMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(0.0f, 0.0f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);
const Material Material::CyanMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(0.0f, 1.0f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);
const Material Material::MagentaMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(1.0f, 0.0f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);
const Material Material::YellowMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(1.0f, 1.0f, 0.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 50.0f);

const Material Material::SkyBlueMaterial(Color::Grey10, Color::SkyBlue, Color::White, 50.0f);
const Material Material::NavyBlueMaterial(Color::Grey10, Color::NavyBlue, Color::White, 50.0f);
const Material Material::YellowGreenMaterial(Color::Grey10, Color::YellowGreen, Color::White, 50.0f);
const Material Material::DarkGreenMaterial(Color::Grey10, Color::DarkGreen, Color::White, 50.0f);
const Material Material::DarkYellowMaterial(Color::Grey10, Color::DarkYellow, Color::White, 50.0f);
const Material Material::MaroonMaterial(Color::Grey10, Color::Maroon, Color::White, 50.0f);
const Material Material::RedBrownMaterial(Color::Grey10, Color::RedBrown, Color::White, 50.0f);
const Material Material::DarkBrownMaterial(Color::Grey10, Color::DarkBrown, Color::White, 50.0f);
const Material Material::GrayBrownMaterial(Color::Grey10, Color::GrayBrown, Color::White, 50.0f);
const Material Material::OrangeMaterial(Color::Grey10, Color::Orange, Color::White, 50.0f);
const Material Material::LavenderMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(0.7f, 0.5f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 10.0f);
const Material Material::PinkMaterial(glm::vec4(0.1f, 0.1f, 0.1f, 1.0f), glm::vec4(0.5f, 0.7f, 1.0f, 1.0f), glm::vec4(1.0f, 1.0f, 1.0f, 1.0f), 10.0f); // ũ ٽ!

Material::Material(void)
{
    Ambient[0] = 0.2f; Ambient[1] = 0.2f; Ambient[2] = 0.2f; Ambient[3] = 1.0f;
    Diffuse[0] = 0.8f; Diffuse[1] = 0.8f; Diffuse[2] = 0.8f; Diffuse[3] = 1.0f;
    Specular[0] = 0.0f; Specular[1] = 0.0f; Specular[2] = 0.0f; Specular[3] = 1.0f;
    Emission[0] = 0.0f; Emission[1] = 0.0f; Emission[2] = 0.0f; Emission[3] = 1.0f;
    Shininess = 1.0f;
}

Material::Material(glm::vec4& ambientAndDiffuse_)
{
    Ambient = ambientAndDiffuse_;
    Diffuse = ambientAndDiffuse_;
    Specular[0] = 0.0f; Specular[1] = 0.0f; Specular[2] = 0.0f; Specular[3] = 1.0f;
    Emission[0] = 0.0f; Emission[1] = 0.0f; Emission[2] = 0.0f; Emission[3] = 1.0f;
    Shininess = 1.0f;
}

Material::Material(glm::vec4& ambientAndDiffuse_, glm::vec4& specular_, float shininess_)
{
    Ambient = ambientAndDiffuse_;
    Diffuse = ambientAndDiffuse_;
    Specular = specular_;
    Emission[0] = 0.0f; Emission[1] = 0.0f; Emission[2] = 0.0f; Emission[3] = 1.0f;
    Shininess = shininess_;
}

Material::Material(glm::vec4& ambient_, glm::vec4& diffuse_, glm::vec4& specular_, float shininess_)
{
    Ambient = ambient_;
    Diffuse = diffuse_;
    Specular = specular_;
    Emission[0] = 0.0f; Emission[1] = 0.0f; Emission[2] = 0.0f; Emission[3] = 1.0f;
    Shininess = shininess_;
}

Material::Material(const Color& ambient_, const Color& diffuse_, const Color& specular_, float shininess_)
{
	Ambient = ambient_;
	Diffuse = diffuse_;
	Specular = specular_;
	Emission[0] = 0.0f; Emission[1] = 0.0f; Emission[2] = 0.0f; Emission[3] = 1.0f;
	Shininess = shininess_;
}

void Material::setAmbientAndDiffuse(glm::vec4& ambientAndDiffuse_)
{
    setAmbient(ambientAndDiffuse_);
    setDiffuse(ambientAndDiffuse_);
}

void Material::setAmbient(glm::vec4& ambient_)
{
    Ambient = ambient_;
}

void Material::setDiffuse(glm::vec4& diffuse_)
{
    Diffuse = diffuse_;
}

void Material::setSpecular(glm::vec4& specular_)
{
    Specular = specular_;
}

void Material::setEmission(glm::vec4& emission_)
{
    Emission = emission_;
}

void Material::setShininess(float shininess_)
{
    Shininess = shininess_;
}


// set material uniform variable
void Material::setUniform(Program* p, std::string materialName) const
{
	p->setUniform(materialName+".Ambient", Ambient);
	p->setUniform(materialName+".Diffuse", Diffuse);
	p->setUniform(materialName+".Specular", Specular);
	p->setUniform(materialName+".Shininess", Shininess);
}

void Material::setUniform(Program* p, std::string materialName)
{
	p->setUniform(materialName + ".Ambient", Ambient);
	p->setUniform(materialName + ".Diffuse", Diffuse);
	p->setUniform(materialName + ".Specular", Specular);
	p->setUniform(materialName + ".Shininess", Shininess);
}

} // namespace
