//////////////////////////////////////////////////////////////////////
// cube3d.cpp: implementation for the Car class (Hierarchical Transformation).
// -Kyoung Shin Park (2016 Spring)
//////////////////////////////////////////////////////////////////////
#include "simpleCube3d.h"
#include "material.h"

namespace moglclass
{
	
SimpleCube3d::SimpleCube3d()
{
	angle = 0.0f; // 0~360
	active = false;
	init();
}

void SimpleCube3d::init()
{
	face = new Quad(glm::vec3(0, 0, 0), glm::vec3(0, 1, 0), glm::vec3(0, 0, -1), 2, 2);

	updateTransform();
}

SimpleCube3d::~SimpleCube3d()
{
	if (face) delete face;
	std::cout << "~SimpleCube3d" << std::endl;
}

void SimpleCube3d::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	Material::YellowMaterial.setUniform(p, "gMaterial");
	glm::mat4 m = model * faceTransform[0];
	p->setUniform("gModel", m);
	if (face) face->draw(); // bottom yellow

	Material::GreenMaterial.setUniform(p, "gMaterial");
	m = model * faceTransform[0] * faceTransform[1];
	p->setUniform("gModel", m);
	if (face) face->draw(); // left green

	Material::CyanMaterial.setUniform(p, "gMaterial");
	m = model * faceTransform[0] * faceTransform[2];
	p->setUniform("gModel", m);
	if (face) face->draw(); // right cyan

	Material::RedMaterial.setUniform(p, "gMaterial");
	m = model * faceTransform[0] * faceTransform[3];
	p->setUniform("gModel", m);
	if (face) face->draw(); // front red

	Material::BlueMaterial.setUniform(p, "gMaterial");
	m = model * faceTransform[0] * faceTransform[4];
	p->setUniform("gModel", m);
	if (face) face->draw(); // back blue

	Material::MagentaMaterial.setUniform(p, "gMaterial");
	m = model * faceTransform[0] * faceTransform[4] * faceTransform[4]; // top magenta (belongs to back face)
	p->setUniform("gModel", m);
	if (face) face->draw(); // top magenta
}

void SimpleCube3d::updateTransform()
{
	faceTransform[0] = glm::translate(glm::mat4(1.0f), glm::vec3(0, -1, 0)); 
	faceTransform[1] = glm::translate(glm::mat4(1.0f), glm::vec3(-1, 0, 0)) * glm::rotate(glm::mat4(1.0f), glm::radians(-angle), glm::vec3(0, 0, 1)) * glm::translate(glm::mat4(1.0f), glm::vec3(1, 0, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(-2, 0, 0));
	faceTransform[2] = glm::translate(glm::mat4(1.0f), glm::vec3(1, 0, 0)) * glm::rotate(glm::mat4(1.0f), glm::radians(angle), glm::vec3(0, 0, 1)) * glm::translate(glm::mat4(1.0f), glm::vec3(-1, 0, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(2, 0, 0));
	faceTransform[3] = glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, 1)) * glm::rotate(glm::mat4(1.0f), glm::radians (-angle), glm::vec3(1, 0, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, -1)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, 2));
	faceTransform[4] = glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, -1)) * glm::rotate(glm::mat4(1.0f), glm::radians(angle), glm::vec3(1, 0, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, 1)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, -2));
}

bool SimpleCube3d::update(float deltaTime)
{
	if (active)
	{
		angle = (float)(deltaTime) * 0.01f;
		if (angle > 90) angle = 90;
		updateTransform();
	}

	return true;
}

} // moglclass namespace